import { implement as AuthImplement } from './index'
import { implement as ACImplement } from './ACL/AccessControl'
import { implement as PMImplement } from './ACL/PermissionManager'
import { ACLRouteProps, isAllowed } from './ACL/AccessControl'
import { blockVisitor, requestLogin } from './channel'
import { useRoutes } from '../PluggableRouter'

/**
 * 当前代码为集成Auth, ACL示例，并无特定作用
 * 可以当作集成代码的模板，去实现具体的业务逻辑
 */

// 实现基本的登录认证
AuthImplement({
  authenticate () {
    const token = 'user-token'
    const isLogin = !!token
    if (isLogin) {
      return Promise.resolve()
    }
    return Promise.reject()
  },
  forwardToLogin () {
    console.warn('forward to login')
    // 集成Channel方法
    requestLogin(location)
  },
})

// 基于以上配置，可选集成访问控制
PMImplement({
  // 希望从服务器读取权限配置信息或从本地读取权限配置信息。不声明则为以下默认逻辑
  // AjaxPermissions () {
  //   // 读取默认配置
  //   return import('./ACL/permissions.default')
  //     .then(({ default: permissions }) => permissions)
  // },
  // 想让ACL自动从路由配置信息中收集资源resources
  getRoutes () {
    return useRoutes()
  }
})
ACImplement({
  getRole () {
    return 'guest'
  }
})
AuthImplement({
  access (route) {
    // 集成ACL模块，进行访问控制
    return isAllowed(route as ACLRouteProps)
  },
  blockVisitor () {
    console.debug('The visitor is not allowed to go', location, '. Then go back.')
    // 集成Channel方法
    blockVisitor()
  }
})
